@@ -1,8 +1,9 @@
 package Po4aBuilder;
 use Module::Build;
 use File::Basename;
-use File::Path;
+use File::Path qw(mkpath rmtree);
 use File::Spec;
+use File::Copy qw(copy);
 use File::stat;
 
 @ISA = qw(Module::Build);
@@ -10,8 +11,9 @@ use File::stat;
 sub ACTION_build {
     my $self = shift;
     $self->depends_on('code');
-    $self->depends_on('po4a_build');
+    $self->depends_on('docs');
     $self->depends_on('distmeta'); # regenerate META.yml
+    $self->depends_on('man');
     $self->depends_on('postats');
 }
 
@@ -25,15 +27,6 @@ sub make_files_writable {
     }
 }
 
-sub ACTION_po4a_build {
-    my $self = shift;
-    $self->depends_on('binpo');
-    $self->make_files_writable("po/pod");
-    system("./share/po4a-build -f po4a-build.conf") && die;
-    File::Path::mkpath( File::Spec->catdir( 'blib', 'manl10n' ), 0, oct(777) );
-    system ("cp -R _build/po4a/man/* blib/manl10n") && die;
-}
-
 sub perl_scripts {
     return ('po4a-gettextize', 'po4a-updatepo', 'po4a-translate',
             'po4a-normalize', 'po4a', 'scripts/msguntypot');
@@ -48,6 +41,7 @@ sub ACTION_binpo {
     my $self = shift;
     my ($cmd, $sources);
 
+    $self->depends_on('code');
     $self->make_files_writable("po/bin");
 
     my @perl_files = sort((perl_scripts(), @{$self->rscan_dir('lib',qr{\.pm$})}));
@@ -55,8 +49,9 @@ sub ACTION_binpo {
     my @all_files = (@perl_files, @shell_files);
     unless ($self->up_to_date(\@all_files, "po/bin/po4a.pot")) {
         print "XX Update po/bin/po4a-perl.pot\n";
+        chdir "po/bin";
         $sources = join ("", map {" ../../".$_ } @perl_files);
-        $cmd = "cd po/bin; xgettext ";
+        $cmd = "xgettext ";
         $cmd .= "--from-code=utf-8 ";
         $cmd .= "-L Perl ";
         $cmd .= "--add-comments ";
@@ -69,7 +64,7 @@ sub ACTION_binpo {
 
         print "XX Update po/bin/po4a-shell.pot\n";
         $sources = join ("", map {" ../../".$_ } @shell_files);
-        $cmd = "cd po/bin; xgettext ";
+        $cmd = "xgettext ";
         $cmd .= "--from-code=utf-8 ";
         $cmd .= "-L shell ";
         $cmd .= "--add-comments ";
@@ -80,8 +75,9 @@ sub ACTION_binpo {
         $cmd .= "-o po4a-shell.pot";
         system($cmd) && die;
 
-        $cmd = "msgcat po/bin/po4a-perl.pot po/bin/po4a-shell.pot -o po/bin/po4a.pot.new";
+        $cmd = "msgcat po4a-perl.pot po4a-shell.pot -o po4a.pot.new";
         system($cmd) && die;
+        chdir "../..";
 
         unlink "po/bin/po4a-perl.pot" || die;
         unlink "po/bin/po4a-shell.pot" || die;
@@ -125,7 +121,7 @@ sub ACTION_binpo {
             print "XX $_ uptodate.\n";
         }
         unless ($self->up_to_date($_,"blib/po/$lang/LC_MESSAGES/po4a.mo")) {
-            File::Path::mkpath( File::Spec->catdir( 'blib', 'po', $lang, "LC_MESSAGES" ), 0, oct(777) );
+            mkpath( File::Spec->catdir( 'blib', 'po', $lang, "LC_MESSAGES" ), 0, oct(755) );
             system("msgfmt -o blib/po/$lang/LC_MESSAGES/po4a.mo $_") && die;
         } 
     }
@@ -138,6 +134,7 @@ sub ACTION_install {
 #    $self->depends_on('build');
     my $mandir = $self->install_sets($self->installdirs)->{'bindoc'};
     $mandir =~ s,/man1$,,;
+    $self->install_path(man => $mandir);
     $self->install_path(manl10n => $mandir);
 
     my $localedir = $mandir;
@@ -150,12 +147,12 @@ sub ACTION_install {
 sub ACTION_dist {
     my ($self) = @_;
 
+    $ENV{PO4AFLAGS} ||= '--force';
     $self->depends_on('test');
     $self->depends_on('binpo');
+    $self->depends_on('manpo');
     $self->depends_on('distdir');
 
-    system("./share/po4a-build --pot-only -f ./po4a-build.conf") and die;
-
     my $dist_dir = $self->dist_dir;
 
     if ( -e "$dist_dir.tar.gz") {
@@ -167,12 +164,119 @@ sub ACTION_dist {
     $self->delete_filetree($dist_dir);
 } 
 
+sub ACTION_manpo {
+    my $self = shift;
+    $self->depends_on('code');
+    $self->make_files_writable("po/pod");
+
+    my $cmd = "PERL5LIB=lib perl po4a "; # Use this version of po4a
+    $cmd .= "--previous ";
+    $cmd .= "--no-translations ";
+    $cmd .= "--msgid-bugs-address po4a-devel\@lists.alioth.debian.org ";
+    $cmd .= "--package-name po4a ";
+    $cmd .= "--package-version ".$self->dist_version()." ";
+    $cmd .= $ENV{PO4AFLAGS}." " if defined($ENV{PO4AFLAGS});
+    $cmd .= "po/pod.cfg";
+    system($cmd)
+        and die;
+}
+
+sub ACTION_man {
+    my $self = shift;
+    $self->depends_on('manpo');
+
+    use Pod::Man;
+    use Encode;
+
+    # Translate binaries manpages
+    my %options;
+    $options{utf8} = 1;
+    my $parser = Pod::Man->new (%options);
+
+    my $manpath  = File::Spec->catdir( 'blib', 'man' );
+    File::Path::rmtree( $manpath, 0, 1);
+
+    my $cmd = "PERL5LIB=lib perl po4a "; # Use this version of po4a
+    $cmd .= $ENV{PO4AFLAGS}." " if defined($ENV{PO4AFLAGS});
+    $cmd .= "--previous po/pod.cfg";
+    system($cmd) and die;
+
+    my $man1path = File::Spec->catdir( $manpath, 'man1' );
+    my $man3path = File::Spec->catdir( $manpath, 'man3' );
+    my $man5path = File::Spec->catdir( $manpath, 'man5' );
+    my $man7path = File::Spec->catdir( $manpath, 'man7' );
+    File::Path::mkpath( $man1path, 0, oct(755) ) or die;
+    File::Path::mkpath( $man3path, 0, oct(755) ) or die;
+    File::Path::mkpath( $man5path, 0, oct(755) ) or die;
+    File::Path::mkpath( $man7path, 0, oct(755) ) or die;
+    copy ( File::Spec->catdir("doc", "po4a-build.conf.5.pod"), $man5path) or die;
+    copy ( File::Spec->catdir("doc", "po4a.7.pod"), $man7path) or die;
+    copy ( File::Spec->catdir("doc", "po4a-runtime.7.pod"), $man7path) or die;
+    foreach $file (perl_scripts()) {
+        $file =~ m,([^/]*)$,;
+        copy($file, File::Spec->catdir($man1path, "$1.1p.pod")) or die;
+    }
+    foreach $file (@{$self->rscan_dir('lib',qr{\.pm$})}) {
+        $file =~ m,([^/]*)$,;
+        copy($file, File::Spec->catdir($man3path, "Locale::Po4a::$1.3pm.pod")) or die;
+    }
+    $self->delete_filetree( File::Spec->catdir("blib", "bindoc") );
+
+    foreach $file (@{$self->rscan_dir($manpath, qr{\.pod$})}) {
+        next if $file =~ m/^man7/;
+        my $out = $file;
+        $out =~ s/\.pod$//;
+        $parser->{name} = $out;
+        $parser->{name} =~ s/^.*\///;
+        $parser->{name} =~ s/^(.*).(1p|3pm|5|7)/$1/;
+        $parser->{section} = $2;
+        if ($parser->{section} ne "3pm") {
+            $parser->{name} = uc $parser->{name};
+        }
+
+        my $lang = $out;
+        $lang =~ s/^blib\/man\/([^\/]*)\/.*$/$1/;
+
+        if ($lang =~ m/man\d/) {
+                $parser->{release} = $parser->{center} = "Po4a Tools";
+        } else {
+                my $command;
+                $command = "msggrep -K -E -e \"Po4a Tools\" po/pod/$lang.po |";
+                $command .= "msgconv -t UTF-8 | ";
+                $command .= "msgexec /bin/sh -c '[ -n \"\$MSGEXEC_MSGID\" ] ";
+                $command .= "&& cat || cat > /dev/null'";
+
+                my $title = `$command 2> /dev/null`;
+                $title = "Po4a Tools" unless length $title;
+                $title = Encode::decode_utf8($title);
+                $parser->{release} = $parser->{center} = $title;
+        }
+        $parser->parse_from_file ($file, $out);
+
+        system("gzip -9 -f $out") and die;
+        unlink "$file" || die;
+    }
+
+    # Install the manpages written in XML DocBook
+    foreach $file (qw(po4a-build.xml po4aman-display-po.xml po4apod-display-po.xml)) {
+        copy ( File::Spec->catdir("share", "doc", $file), $man1path) or die;
+    }
+    foreach $file (@{$self->rscan_dir($manpath, qr{\.xml$})}) {
+        if ($file =~ m,(.*/man(.))/([^/]*)\.xml$,) {
+            my ($outdir, $section, $outfile) = ($1, $2, $3);
+            system("xsltproc -o $outdir/$outfile.$section --nonet http://docbook.sourceforge.net/release/xsl/current/manpages/docbook.xsl $file") and die;
+            system ("gzip -9 -f $outdir/$outfile.$section") and die;
+        }
+        unlink "$file" || die;
+    }
+}
+
 sub ACTION_postats {
     my $self = shift;
     $self->depends_on('binpo');
-    $self->postats("po/bin");
-    $self->postats("po/pod");
-    $self->postats("po/www") if -d "po/www";
+    $self->postats( File::Spec->catdir("po", "bin"));
+    $self->postats( File::Spec->catdir("po", "pod"));
+    $self->postats( File::Spec->catdir("po", "www")) if -d File::Spec->catdir("po", "www");
 }
 
 sub postats {
