@@ -3,7 +3,7 @@
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
 #
-# This module converts POD to PO file, so that it becomes possible to 
+# This module converts POD to PO file, so that it becomes possible to
 # translate POD formatted documentation. See gettext documentation for
 # more info about PO files.
 
@@ -37,6 +37,8 @@ sub new {
         $modname = 'BibTex';
     } elsif ($module eq 'tex') {
         $modname = 'TeX';
+    } elsif ($module eq 'asciidoc') {
+        $modname = 'AsciiDoc';
     } else {
         $modname = ucfirst($module);
     }
@@ -45,9 +47,9 @@ sub new {
         if ($@) {
             my $error=$@;
             warn wrap_msg(gettext("Unknown format type: %s."), $module);
-	    warn wrap_mod("po4a::chooser",
-		gettext("Module loading error: %s"), $error)
-	      if defined $options{'verbose'} && $options{'verbose'} > 0;
+            warn wrap_mod("po4a::chooser",
+                gettext("Module loading error: %s"), $error)
+              if defined $options{'verbose'} && $options{'verbose'} > 0;
             list(1);
         }
     }
@@ -56,23 +58,24 @@ sub new {
 
 sub list {
     warn wrap_msg(gettext("List of valid formats:")
-#	."\n  - ".gettext("bibtex: BibTex bibliography format.")
-	."\n  - ".gettext("dia: uncompressed Dia diagrams.")
-	."\n  - ".gettext("docbook: Docbook XML.")
-	."\n  - ".gettext("guide: Gentoo Linux's xml documentation format.")
-#	."\n  - ".gettext("html: HTML documents (EXPERIMENTAL).")
-	."\n  - ".gettext("ini: .INI format.")
-	."\n  - ".gettext("kernelhelp: Help messages of each kernel compilation option.")
-	."\n  - ".gettext("latex: LaTeX format.")
-	."\n  - ".gettext("man: Good old manual page format.")
-	."\n  - ".gettext("pod: Perl Online Documentation format.")
-	."\n  - ".gettext("sgml: either debiandoc or docbook DTD.")
-	."\n  - ".gettext("texinfo: The info page format.")
-	."\n  - ".gettext("tex: generic TeX documents (see also latex).")
-	."\n  - ".gettext("text: simple text document.")
-	."\n  - ".gettext("wml: WML documents.")
-	."\n  - ".gettext("xhtml: XHTML documents.")
-	."\n  - ".gettext("xml: generic XML documents (see also docbook).")
+#       ."\n  - ".gettext("bibtex: BibTex bibliography format.")
+        ."\n  - ".gettext("asciidoc: AsciiDoc format.")
+        ."\n  - ".gettext("dia: uncompressed Dia diagrams.")
+        ."\n  - ".gettext("docbook: DocBook XML.")
+        ."\n  - ".gettext("guide: Gentoo Linux's XML documentation format.")
+#       ."\n  - ".gettext("html: HTML documents (EXPERIMENTAL).")
+        ."\n  - ".gettext("ini: INI format.")
+        ."\n  - ".gettext("kernelhelp: Help messages of each kernel compilation option.")
+        ."\n  - ".gettext("latex: LaTeX format.")
+        ."\n  - ".gettext("man: Good old manual page format.")
+        ."\n  - ".gettext("pod: Perl Online Documentation format.")
+        ."\n  - ".gettext("sgml: either DebianDoc or DocBook DTD.")
+        ."\n  - ".gettext("texinfo: The info page format.")
+        ."\n  - ".gettext("tex: generic TeX documents (see also latex).")
+        ."\n  - ".gettext("text: simple text document.")
+        ."\n  - ".gettext("wml: WML documents.")
+        ."\n  - ".gettext("xhtml: XHTML documents.")
+        ."\n  - ".gettext("xml: generic XML documents (see also docbook).")
     );
     exit shift;
 }
@@ -87,20 +90,20 @@ __END__
 
 =head1 NAME
 
-Locale::Po4a::Chooser - Manage po4a modules
+Locale::Po4a::Chooser - manage po4a modules
 
 =head1 DESCRIPTION
 
-Locale::Po4a::Chooser is a module to manage po4a modules. Before, all po4a
+Locale::Po4a::Chooser is a module to manage po4a modules. Previously, all po4a
 binaries used to know all po4a modules (pod, man, sgml, etc). This made the
-add of a new module boring, to make sure the documentation is synchronized
+addition of a new module boring, to make sure the documentation is synchronized
 in all modules, and that each of them can access the new module.
 
 Now, you just have to call the Locale::Po4a::Chooser::new() function,
 passing the name of module as argument.
 
 You also have the Locale::Po4a::Chooser::list() function which lists the
-available format and exits on the value passed as argument.
+available formats and exits on the value passed as argument.
 
 =head1 SEE ALSO
 
@@ -108,9 +111,9 @@ available format and exits on the value passed as argument.
 
 =item About po4a:
 
-L<po4a(7)|po4a.7>, 
+L<Locale::Po4a::Po(3pm)>,
 L<Locale::Po4a::TransTractor(3pm)>,
-L<Locale::Po4a::Po(3pm)>
+L<po4a(7)|po4a.7>
 
 =item About modules:
 
